unit Module2;

interface

uses
  Windows, olectnrs, main;

const
MM_HIMETRIC = 3;
MM_TWIPS = 6;
MM_ANISOTROPIC = 8;
GetPrintingOffset = 13;

Type METAFILEPICT = Record
    mm : LongInt;
    xext : LongInt;
    yext : LongInt;
    hMF : LongInt;
End;

Type POINTAPI = Record
    X : LongInt;
    Y : LongInt;
End;

Type SIZEAPI = Record
    xs : LongInt;
    ys : LongInt;
End;

Type RECTAPI = Record
    nLeft : LongInt;
    nTop : LongInt;
    nWidth : LongInt;
    nHeight : LongInt;
End;

Procedure memmove(hpvDest : METAFILEPICT; hpvSource : Pointer; cbCopy : LongInt); stdcall;
 external  'MSVCRT40';
Function LPtoDPr(hdc : LongInt; lpPoints : RECTAPI; nCount : LongInt) : LongInt; stdcall;
 external 'gdi32' name 'LPtoDP';

implementation

Function DrawMetaFile(MFPict : METAFILEPICT; nZoom, hdc, nLeft, nTop, nWidth, nHeight : Integer) : Boolean;
    { This function prints a metafile. In TX Publisher, it is used to print
     the contents of OLE controls.
     Parameters:
       MFPict      is the metafile handle. Use GetMetaFile() to
                   get the metafile from an OLE control.
       nZoom       is an optional Zoom factor
       hDC         is the printer's display context handle (hDC property)
       nLeft,
       nTop,
       nWidth,
       nHeight     define the rectangle (in twips) where the metafile
                   is printed. The printer's printing offset is
                   subtracted, i.e a point at (0, 0) is printed exactly
                   at the upper left corner of the page.}
Var
    rMap : RECTAPI;
    ptPrintOffs : POINTAPI;
    oldmapmode : Integer;   // Stores the old map mode.
    picleft : Integer;
    pictop : Integer;
    picwidth : Integer;     // Holds picture width in pixels.
    picheight : Integer;    // Holds picture height in pixels.
    retvalue : LongInt;     // Holds API return values.
    retval2 : LongBool;    // Holds API return values.
    oldviewportext : SIZEAPI;   // Stores the old viewport origin.
    oldviewportorg : POINTAPI;  // Stores the old viewport extension.
Label
     DMFError;
begin
    DrawMetaFile := True;

    // Calculate the size in pixels.
    oldmapmode := SetMapMode(hdc, MM_TWIPS);

    rMap.nLeft := nLeft;
    rMap.nTop := nTop;
    rMap.nWidth := nWidth;
    rMap.nHeight := nHeight;

    // Convert logical coordinates to printer device coordinates
    If (LPtoDPr(hdc, rMap, 2)) = 0 Then begin
        // ReEstablish oldmapmode and exit with error.
        retvalue := SetMapMode(hdc, oldmapmode);
        GoTo DMFError;
    End;

    // Subtract printing offset
    If (Escape(hdc, GetPrintingOffset, 0, '0', @ptPrintOffs) > 0) Then begin
        rMap.nLeft := rMap.nLeft - ptPrintOffs.X;
        rMap.nTop := rMap.nTop + ptPrintOffs.Y;
    End;

    // Now set the mapmode to ANISOTROPIC to match the
    // mapmode of the metafile.
    retvalue := SetMapMode(hdc, MM_ANISOTROPIC);

    // Retrieve the converted width and height.
    // Some values will be negative, so use Abs.
    picleft := Abs(rMap.nLeft);
    pictop := Abs(rMap.nTop);
    picwidth := Abs(rMap.nWidth);
    picheight := Abs(rMap.nHeight);

    // Scale to Zoom factor.
    picleft := picleft * nZoom;
    pictop := pictop * nZoom;
    picwidth := picwidth * nZoom;
    picheight := picheight * nZoom;

    // Set the viewport to match control size and zoom.
    retval2 := SetViewPortExtEx(hdc, picwidth, picheight, (@oldviewportext));
    retval2 := SetViewPortOrgEx(hdc, picleft, pictop, @oldviewportorg);

    // Play the metafile to the hdc.
    If PlayMetaFile(hdc, MFPict.hMF) = False Then GoTo DMFError;

    // ReEstablish old viewport and map mode for the hdc.
    retval2 := SetViewPortExtEx(hdc, oldviewportext.xs, oldviewportext.ys, PSize(0));
    retval2 := SetViewPortOrgEx(hdc, oldviewportorg.X, oldviewportorg.Y, PPoint(0));
    retvalue := SetMapMode(hdc, oldmapmode);

    Exit;

DMFError:
    DrawMetaFile := False;

End;

Procedure GetMetaFile(MFPict : METAFILEPICT; olectrl : TOleContainer);
    // Get the metafile from an OLE object.
var
    hGlbMem : LongInt;         // Handle to Global Memory Object.
    APISuccess : LongBool;   // Return value for errors (if any)
    lpMem : Pointer;           // Long Pointer to Memory.

Label
     gmf_err;
begin
    //On Err GoTo gmf_err;


    // OLE Control must be activated to get MetaFile.
    olectrl.DoVerb(-2);

    // Tell the OLE Control what format we want.
    olectrl.ClassNameIs ('CF_METAFILEPICT');

    // Retrieve the Global Memory Handle from Data Property.
    // Copy low 2 bytes to hGlbMem.
    // hmemcpy2 lets us copy the unsigned integer part of tempdata.
    hGlbMem := olectrl.Handle;

    // Retrieve pointer to Global Memory.
    lpMem := GlobalLock(hGlbMem);

    // Copy Metafile to MFPict.
    memmove(MFPict, lpMem, Sizeof(MFPict));

    // Release pointer to Global Memory.
    APISuccess := GlobalUnlock(hGlbMem);

    // Close OLE Server
    olectrl.Close;

    Exit;

gmf_err:
    MessageBox(frmMain.Handle, '_ERROR_', 'ERROR', MB_OK);  //Err.Description
    Exit;
End;


end.
